#----------------------------------------------------------------------
compare.n.clusters <- function(cObjList, CL=3, minfreq=2)  {
  sL <- 0

  ind.empty <- NULL
  for( n in 1:length(cObjList) )  {
    if (length(cObjList[[n]]$size) ==0) {
      print(paste( "Cluster #", n, " is an empty one", sep=""))
      ind.empty <- c(ind.empty,n)
      return(cObjList[[n]])
    }
  }
  
  if (  length(ind.empty) >0) cObjList <- cObjList[-ind.empty]
  
  for( n in 1:length(cObjList) )  {
    sL <- max(sL, cObjList[[n]]$end)
  }

  sSum <- integer(sL)
  for( n in 1:length(cObjList) )  {
    sN <- integer(sL)
    for( i in 1:length(cObjList[[n]]$size) )  {
      sN[cObjList[[n]]$begin[i]:cObjList[[n]]$end[i]] <- 1
    }
    sSum <- sSum + sN
  }
  
  sSum[sSum < minfreq] <- 0
  sSum[sSum > 0] <- 1
  
  d1 <- get.clusters2(sSum,CL)
  d1$minsize <- CL
  d1$minfreq <- minfreq
  
  if( length(d1$size) == 0 )
    d1$size <- d1$begin <- d1$end <- NULL  
  return (d1)
}


##---------------------------------------------------------------------
find.strict.nonoverlapping.clusters <- function(cObj1, cObj2)  {
  if( length(cObj1$size) == 0 || length(cObj2$size) == 0 ) {
    d1 <- d2 <- list(size = NULL)
  }
  else  {
    s1 <- s2 <- integer(max(cObj1$end, cObj2$end))
    for( n in 1:length(cObj1$size) )
      s1[cObj1$begin[n]:cObj1$end[n]] <- 1

    for( n in 1:length(cObj2$size) )
      s2[cObj2$begin[n]:cObj2$end[n]] <- 1
    
    sSum <- s1 + s2
    ind.d1.okay <- ind.d2.okay <- NULL

    for( n in 1:length(cObj1$size) )  {
      if( sum(sSum[cObj1$begin[n]:cObj1$end[n]]) == cObj1$size[n] )  {
        ind.d1.okay <- c(ind.d1.okay, n)
      }
    }

    for( n in 1:length(cObj2$size) )  {
      if( sum(sSum[cObj2$begin[n]:cObj2$end[n]]) == cObj2$size[n] )  {
        ind.d2.okay <- c(ind.d2.okay, n)
      }
    }

    d1 <- cObj1
    d1$begin <- d1$begin[ind.d1.okay]
    d1$end <- d1$end[ind.d1.okay]
    d1$size <- d1$size[ind.d1.okay]

    d2 <- cObj2
    d2$begin <- d2$begin[ind.d2.okay]
    d2$end <- d2$end[ind.d2.okay]
    d2$size <- d2$size[ind.d2.okay]
  }
  
  return(list(d1 = d1, d2 = d2))
}


##-------------------------------------------------------------------------
## uses vectorized form (faster than get.clusters() for large vectors)
get.clusters2 <- function(x,CL)  {
  temp <- which(diff(x) != 0)
  begin <- c(1, temp + 1)
  end <- c(temp, length(x))
  size <- end - begin + 1

  begin <- begin[size >= CL]
  end <- end[size >= CL]
  size <- size[size >= CL]

  size <- size[x[end] != 0]
  begin <- begin[x[end] != 0]
  end <- end[x[end] != 0]
  
  return (list(size=size,begin=begin,end=end))
}


##-------------------------------------------------------------------------
get.cutoff.clusters <- function(V, cutoff, minsize)  {
  return(get.clusters2(as.numeric(V >=cutoff), minsize))
}


##-------------------------------------------------------------------------
get.imd.clusters <- function(V, cutoff.min, cutoff.max, minsize)  {
  V2 <- V
  V2[V > cutoff.max] <- -999
  return(get.clusters2(as.numeric(V2 > cutoff.min), minsize))
}


##-------------------------------------------------------------------------
get.nearby.genes2 <- function(sPos, ePos, cl, GeneInfo) {

  gene <- dist.gene <- NULL

  exon.sPos <- GeneInfo[,4]
  exon.ePos <- GeneInfo[,5]
  idVector <- as.character(GeneInfo[,10])
    
  for (i in 1:length(cl$size)) {
    cl.b <- sPos[i]
    cl.e <- ePos[i]

    s1 <- (cl.b+cl.e)/2 - exon.sPos # begining of exons
    s2 <- which(min(abs(s1))==abs(s1))   # closest exon
    if (length(s2)>1) (s2 <- round(mean(s2)))

    current.how.far <- 1e10
    for (j in max(1,(s2-20)):(s2+20)) {
      exon.b <- exon.sPos[j]
      exon.e <- exon.ePos[j]
      id <- idVector[j]

      if (exon.e<cl.b) {  # exon is before the cluster
        how.far <- cl.b-exon.e
      } else if (exon.b>cl.e) {  # exon is after the cluster
        how.far <- exon.b-cl.e
      }
      else {
        current.how.far <- 0
        gene.id <- id
        break;
      }
      if (how.far < current.how.far) {
        current.how.far <- how.far
        gene.id <- id
      }
    }

    dist.gene[i] <- current.how.far
    gene[i] <- gene.id
    cat(".")
  }
  cat("\n")
  return (list(dist.gene=dist.gene,gene=gene))
}


##----------------------------------------------------------------------
get.gene.list2 <- function(sPos, ePos, DMEL, na = "")
{
  print("Consider to use get.gt.list to get gene list at certain location!")
  DMEL.chr <- DMEL[DMEL[,3] == "exon", ]
  DMEL.chr <- DMEL.chr[order(DMEL.chr[,4], DMEL.chr[,5]), ]
  transcript.ID <- as.character(DMEL.chr[,10])
  gene.ID <- sub("-R[A-Z]", "", transcript.ID)

  gV <- tV <- rep.int(na, length(sPos))
  for(i in 1:length(sPos))  {
    ind <- which(DMEL.chr[,4] < ePos[i] & DMEL.chr[,5] > sPos[i])
    if(length(ind) > 0)  {
      gV[i] <- paste(unique(gene.ID[ind]), collapse = ", ")
      tV[i] <- paste(unique(transcript.ID[ind]), collapse = ", ")
    }
  }
  
  return(list(gene = gV, transcript = tV))
}


##----------------------------------------------------------------------
get.gt.list <- function(sPos, ePos, gtObj, na = "")
{
  gV <- tV <- gS <- tS <- rep.int(na, length(sPos))
  for(i in 1:length(sPos))  {
    ind.gene <- which(gtObj$gene.sPos < ePos[i] &
                      gtObj$gene.ePos > sPos[i])
    ind.transcript <- which(gtObj$transcript.sPos < ePos[i] &
                            gtObj$transcript.ePos > sPos[i])
    if(length(ind.gene) > 0)  {
      gV[i] <- paste(gtObj$gene[ind.gene], collapse = ", ")
      tV[i] <- paste(gtObj$transcript[ind.transcript], collapse = ", ")
      gS[i] <- paste(gtObj$strand.gene[ind.gene], collapse = ", ")
      tS[i] <- paste(gtObj$strand.transcript[ind.transcript], collapse = ", ")
    }
  }

  return(list(gene = gV, transcript = tV,
              strand.gene = gS, strand.transcript = tS))
}


##----------------------------------------------------------------------
get.gt.boundaries <- function(DMEL.chr)
{
  ## get gene boundaries
  DMEL.chr <- DMEL.chr[DMEL.chr[,3] == "exon", ]
  DMEL.chr <- DMEL.chr[order(DMEL.chr[,4], DMEL.chr[,5]), ]
  transcript.ID <- as.character(DMEL.chr[,10])
  gene.ID <- sub("-R[A-Z]", "", transcript.ID)

  unique.gene <- sort(unique(gene.ID))
  ##gene.sPos <- gene.ePos <- integer(length(unique.gene))
  ##strand.gene <- character(length(unique.gene))

  unique.transcript <- sort(unique(transcript.ID))
  ##transcript.sPos <- transcript.ePos <- integer(length(unique.transcript))
  ##strand.transcript <- character(length(unique.transcript))
 
  ##for(i in 1:length(unique.gene))  {
  ##  ind <- which(gene.ID == unique.gene[i])
  ##  gene.sPos[i] <- DMEL.chr[min(ind),4]
  ##  gene.ePos[i] <- DMEL.chr[max(ind),5]
  ##  strand.gene[i] <- DMEL.chr[min(ind),7]
  ##}

  ##for(i in 1:length(unique.transcript))  {
  ##  ind <- which(transcript.ID == unique.transcript[i])
  ##  transcript.sPos[i] <- DMEL.chr[min(ind),4]
  ##  transcript.ePos[i] <- DMEL.chr[max(ind),5]
  ##  strand.transcript[i] <- DMEL.chr[min(ind),7]
  ##}

  strand.gene <- DMEL.chr[match(unique.gene, gene.ID), 7]
  strand.transcript <- DMEL.chr[match(unique.transcript, transcript.ID), 7]
  
  gene.sPos <- tapply(DMEL.chr[,4], as.factor(gene.ID), min)
  gene.ePos <- tapply(DMEL.chr[,5], as.factor(gene.ID), max)
  transcript.sPos <- tapply(DMEL.chr[,4], as.factor(transcript.ID), min)
  transcript.ePos <- tapply(DMEL.chr[,5], as.factor(transcript.ID), max)
    
  return(list(gene = unique.gene, strand.gene = strand.gene,
              gene.sPos = gene.sPos, gene.ePos = gene.ePos,
              transcript = unique.transcript,
              strand.transcript = strand.transcript,
              transcript.sPos = transcript.sPos,
              transcript.ePos = transcript.ePos ))            
}

##----------------------------------------------------------------------
calcFDR <- function(x, cutoffV, mcsV, nperm, verbose = TRUE, libDir = "src")
{
  if(!is.loaded("calc_fdr"))
    dyn.load(file.path(libDir, "cluster_FDR"))

  res <- .C("calc_fdr",
            x = as.numeric(x),
            nx = as.integer(length(x)),
            cutoffV = as.numeric(cutoffV),
            ncutoffV = as.integer(length(cutoffV)),
            mcsV = as.integer(mcsV),
            nmcsV = as.integer(length(mcsV)),
            nperm = as.integer(nperm),
            verbose = as.integer(verbose == TRUE),
            fdrM = numeric(length(cutoffV)*length(mcsV))
            )[[9]]
  dim(res) <- c(length(cutoffV), length(mcsV))
  rownames(res) <- cutoffV
  colnames(res) <- mcsV
  return(res)
}

# -------------------------------------
mad.diff	<- function(data){
	if (is.matrix(data)){		
		mad.diff.data	<- numeric(ncol(data))
		for (i in 1:ncol(data)){
			mad.diff.data[i]	<- 4*mad(diff(data[,i]))/sqrt(2)
		}
	}else{
		mad.diff.data	<- 4*mad(diff(data))/sqrt(2)
	}
	return(mad.diff.data)
}

mad.diff_lag	<- function(data,lag=10){
  if (is.matrix(data)){		
    mad.diff.data	<- numeric(ncol(data))
    names(mad.diff.data) <- colnames(data)
    for (i in 1:ncol(data)){
      mad.diff.data[i]	<- mad(diff(data[,i],lag=lag))/sqrt(2)
    }
  }else{
    mad.diff.data	<- mad(diff(data,lag=lag))/sqrt(2)
  }
  return(mad.diff.data)
}

mad.diff_nlag <- function(data,lag=25){
  if (is.matrix(data)){		
    mad.diff.n	<- matrix(NA, nrow=lag,ncol(data))
    colnames(mad.diff.n) <- colnames(data)
    for (i in 1:lag){
      mad.diff.n[i,]	<- mad.diff_lag(data,lag=i)
    }
  }else{
    mad.diff.n <- numeric(lag)
    for (i in 1:lag){
      mad.diff.n[i]	<- mad.diff_lag(data,lag=i)
    }
  }
  return(mad.diff.n)
}

#----------------
mad.2L	<- function(data){
	if (is.matrix(data)){		
		mad.data	<- numeric(ncol(data))
		for (i in 1:ncol(data)){
			mad.data[i]	<- mad(data[index.2L.F,i])
		}
	}else{
		mad.data	<- mad(data[index.2L.F])
	}
	return(mad.data)
 }

sd.2L	<- function(data){
	if (is.matrix(data)){		
		sd.data	<- numeric(ncol(data))
		for (i in 1:ncol(data)){
			sd.data[i]	<- sd(data[index.2L.F,i])
		}
	}else{
		sd.data	<- sd(data[index.2L.F])
	}
	return(sd.data)
 }



Cutoff_quantile	<- function(data,top=0.05){
	if (is.matrix(data)){		
		quantile.data	<- numeric(ncol(data))
		for (i in 1:ncol(data)){
			quantile.data[i]	<- quantile(data[,i],(1-top))
		}
	}else{
		quantile.data	<- quantile(data,(1-top))
	}
	return(quantile.data)
}

sm	<- function(data){
	sm.data <- data
	if (is.matrix(data)){
		for (i in 1:ncol(data)){
			sm.data[,i]	<- runmed(data[,i],k=7)
		}
	}else{
		sm.data 	<- runmed(data,k=7)
	}
	return(sm.data)
}

#======================================
RescaleData	<- function(Data){
	Data1 <- Data
	factor <- mad.diff(Data)
	if (is.matrix(Data)){
		n <- ncol(Data)
		for ( i in 1:n){
			Data1[,i] 	<- Data[,i]/factor[i]
		}

	}else{
		Data1	<- Data/factor
	}
return(Data1)
}

#==========================================
check.3CES.X <- function(cObj){
	print("Check whether 3 entry sites (on roX1, roX2, and 18D11) are in the Common Clusters")
	for (i in 1:3){
#		ind.Rox <- 	(2+roX[i,"sInd"]):(2+roX[i,"eInd"])
		ind.Rox <- 	roX[i,"sInd"]:roX[i,"eInd"]
		for (j in 1:length(cObj$size)){
			ind.BS 	<- cObj$begin[j]:cObj$end[j]
			if (length(intersect(ind.Rox,ind.BS))>0){
				print(paste(rownames(roX)[i]," is in clusters # ", j, sep=""))
			}
		}
	}
}
#=================================
rank.clusters	<- function(cObj,Data,Cutoff){
	Data1 <- as.matrix(Data)
	n <- ncol(Data1)
	for (i in 1:n)
		Data1[,i] = Data[,i]-Cutoff[i]

	Data1 <- (Data1+abs(Data1))/2
	len <-length(cObj$size)
	score <- numeric(len)
	for (j in 1:len){
		k <- cObj$begin[j]:cObj$end[j]
		score[j] <- sum(Data1[k,])
	}

	ind <- order(score,decreasing=TRUE)
	return(list(size=cObj$size[ind],begin=cObj$begin[ind],end=cObj$end[ind],
			score=score[ind]))
}

#=================================
get.DNAsequence	<- function(cObj, chr=chr.X, ngAnnot=ngAnnot.X.F, minsize=3, extraDNA=0,minlen=0,
                            seq0=0,index.subset=NULL){
	temp=list()
        if (is.null(index.subset)) {
          index <- which(cObj$size>=minsize)
        }else{
          index <- index.subset
        }
	for (j in 1:length(index)){
		i = index[j]
		sPos	<- ngAnnot[cObj$begin[i],1]-extraDNA
		ePos	<- ngAnnot[cObj$end[i],2]-1+extraDNA
		len	<- (ePos-sPos+1)
		if (len < minlen){
			mPos	<- floor((sPos+ePos)/2)
			sPos	<- mPos-floor(minlen/2)
			ePos	<- sPos+minlen-1
		}
		cyto <- ""
 		if (chr == chr.X) cyto <- get.cytoBand(CytoBand.chr=cytoBand.X,sPos,ePos)
 		if (chr == chr.2L) cyto <- get.cytoBand(CytoBand.chr=cytoBand.2L,sPos,ePos)
		temp.name <- paste("> seq#",i+seq0,
				" at ",cyto,
				"  Position[",sPos,":",ePos,"] ",
				 " length=",ePos-sPos+1, sep="")
		temp.DNA	<- substr(chr, sPos,ePos)	
		temp[[j]] <-  rbind(temp.name,temp.DNA)
	}
	return(temp)
}

get.DNAsequence.X	<- function(cObj, minsize=3, extraDNA=0,minlen=0, seq0=0,index.subset=NULL ){
  return(get.DNAsequence(cObj, chr=chr.X, ngAnnot=ngAnnot.X.F,minsize, extraDNA,minlen, seq0, index.subset))
}

get.DNAsequence.2L	<- function(cObj, minsize=3, extraDNA=0,minlen=0, seq0=0,index.subset=NULL ){
  return(get.DNAsequence(cObj, chr=chr.2L, ngAnnot=ngAnnot.2L.F,minsize, extraDNA,minlen, seq0,index.subset))
}

#gtObj <- gtObj.X
#CG <- gtObj.X$gene[3:5]

get.DNA.genes <- function(CG, chr=chr.X, gtObj=gtObj.X, extraDNA=0, minlen=0,seq0=0){
  temp=list()
  index <- which(CG %in% gtObj$gene)
  if (length(CG) != length(index)) {
    print("There is some genes are not in the corresponding chromosome ")
    print(length(CG)-length(index))
    print(CG[-index])
  }
  
  for (j in 1:length(index)){
    i = index[j]
    gene <- CG[i]
    sPos	<- gtObj$gene.sPos[gene]-extraDNA
    ePos	<- gtObj$gene.ePos[gene]+extraDNA
    len	<- (ePos-sPos+1)
    if (len < minlen){
      mPos	<- floor((sPos+ePos)/2)
      sPos	<- mPos-floor(minlen/2)
      ePos	<- sPos+minlen-1
    }
    cyto <- ""
    if (chr == chr.X) cyto <- get.cytoBand(CytoBand.chr=cytoBand.X,sPos,ePos)
    if (chr == chr.2L) cyto <- get.cytoBand(CytoBand.chr=cytoBand.2L,sPos,ePos)
    temp.name <- paste("> seq#",i+seq0,"[",gene,"]",
                       " at ",cyto,
                       "  Position[",sPos,":",ePos,"] ",
                       " length=",ePos-sPos+1, sep="")
    temp.DNA	<- substr(chr, sPos,ePos)	
    temp[[j]] <-  rbind(temp.name,temp.DNA)
  }
  return(temp)  
}

get.DNA.genes.X <- function(CG, extraDNA=0, minlen=0,seq0=0){
  return( get.DNA.genes(CG=CG, chr=chr.X, gtObj=gtObj.X, extraDNA=extraDNA, minlen=minlen,seq0=seq0))
}

get.DNA.genes.2L <- function(CG, extraDNA=0, minlen=0,seq0=0){
  return( get.DNA.genes(CG=CG, chr=chr.2L, gtObj=gtObj.2L, extraDNA=extraDNA, minlen=minlen,seq0=seq0))
}

get.cObj.gap <- function(cObj){
  n <- length(cObj$size)
  if (n<2){
    print("There should be more than 2 clusters")
    return()
  }
  begin <- cObj$end[-n]+1
  end <- cObj$begin[-1]-1
  size <- end-begin+1
  cObj.gap <- list(begin=begin,end=end,size=size)
  return(cObj.gap)
}

get.subset.cObj <- function(cObj, ind.subset){
  begin <- cObj$begin[ind.subset]
  end <- cObj$end[ind.subset]
  size <- cObj$size[ind.subset]
  return(list(begin=begin,end=end,size=size))
}

#=================================
matchDNAPattern.good <- function(pattern,x){
  # get rid of motifs which are too close to the precious one.
  # most likely they are due to NNN or other repeats.
  nc <- nchar(pattern)
  if (class(x)!="list"){
    	ms0 <- matchDNAPattern(pattern,x)
    	ind <- which(anySameLetter(ms0,"N"))
	if (length(ind)>0) ms <- as.matrix(ms0[-ind]) else ms <- as.matrix(ms0)

	if ((nrow(ms)>1) && (length(which(diff(ms)[,1] < nc))>0)){
		ms.good <- matrix(ms[-(which(diff(ms)[,1] <nc)+1),],ncol=2)
	}else{
		ms.good <- ms
	} 
	print(paste(nrow(ms.good), " appearing times of the motif",sep=""))
	if (nrow(ms.good)==0)	ms.good <- ms.good[,1] 	else ms.good <- (ms.good[,1]+ms.good[,2])/2				
  }else{
    ms.good	<- list()
    nl <- length(x)
    ms.num  <- integer(nl)
    for (i in 1:nl){
      ms0 <- matchDNAPattern(pattern,x[[i]]$seq)
      ind <- which(anySameLetter(ms0,"N"))
 	if (length(ind)>0) ms <- as.matrix(ms0[-ind]) else ms <- as.matrix(ms0)
	if ((nrow(ms)>1) && (length(which(diff(ms)[,1] < nc))>0)){
		ms.good[[i]] <- matrix(ms[-(which(diff(ms)[,1] <nc)+1),],ncol=2)
	}else{
		ms.good[[i]] <- ms
	} 
#    print(i)
     ms.num[i] 	<- nrow(ms.good[[i]])
     if (nrow(ms.good[[i]])==0)	ms.good[[i]] <- ms.good[[i]][,1] 	else ms.good[[i]] <- (ms.good[[i]][,1]+ms.good[[i]][,2])/2

    }
    print(paste(sum(ms.num), " appearing times in ", length(which(ms.num>0)), " out of ", nl,sep=""))
  }
  return(ms.good)
}

# =================================
get.DistDistr	<- function(data0,data1,cutoff=500){
  delta <- list()
  cutoff <- abs(cutoff)
  if (length(data0)>0 && length(data1)>0){
	for (i in 1:length(data0)){
		data <- data1-data0[i]
		data.abs	<- abs(data)
		ind1 <- which(data.abs <=cutoff)
		ind2	<- which(data.abs > 0)
		ind <- intersect(ind1,ind2)
		delta[[i]] <- data[ind]
	}
  }
  return(unlist(delta))
}

plot.DistDistr	<- function(dist1, dist2, xlim1=500, xlim2=100, xlim3=10, main="",sub=""){
	par(mfrow=c(2,3))
	xlab="distance"
	hist(dist1,breaks=-500.5:500.5,main="",	xlab=xlab,xlim=c(-xlim1,xlim1))
	hist(dist1,breaks=-500.5:500.5,main=main,	xlab=xlab,xlim=c(-xlim2,xlim2))
	hist(dist1,breaks=-500.5:500.5,main=sub, 	xlab=xlab,xlim=c(-xlim3,xlim3))

	hist(dist2,breaks=-500.5:500.5,main="",					xlab=xlab,xlim=c(-xlim1,xlim1))
	hist(dist2,breaks=-500.5:500.5,main="between complimentaries",	xlab=xlab,xlim=c(-xlim2,xlim2))
	hist(dist2,breaks=-500.5:500.5,main="",					xlab=xlab,xlim=c(-xlim3,xlim3))
} 

# =================================
get.DistDistr2	<- function(data0,data1,cutoff=500,dir="+"){
  delta <- list()
  j=1
  if (length(data0)>0 && length(data1)>0){
	if (dir=="+"){
		for (i in 1:length(data0)){
			 data <- data1-data0[i]
			Dist= data[which(abs(data) <=cutoff)]
			if (length(Dist)>0){
				delta[[j]] <- list(Pos=data0[i], Dist= Dist)
				j=j+1
			}
		}
	}else{
		for (i in 1:length(data0)){
			 data <- -(data1-data0[i])
			Dist= data[which(abs(data) <=cutoff)]
			if (length(Dist)>0){
				delta[[j]] <- list(Pos=data0[i], Dist= Dist)
				j=j+1
			}
		}
	}
  }
  return(delta)
}


plot.DistDistr2	<- function(dist, xlim=200, main="",sub="", cObj="", ngAnnot=ngAnnot.X.F){
#	par(oma=c(1,1,1,1))
	xlab="distance"
#	plot.window( xlim=c(-xlim,xlim),ylim=c(1,(length(dist))))
	plot(x=0,y=0,main=main, xlim=c(-xlim,xlim),ylim=c(1,length(dist)),xlab=xlab,ylab="",pch=".",cex=0.5)
	abline(v=0,lty=2)
	for (i in 1:length(dist)){
		abline(h=i)
		if (length(cObj$size)>0){
			ind <- max(which(ngAnnot[,1]<=dist[[i]]$Pos))
			for (j in 1:length(cObj$size)){
				if (is.element(ind, cObj$begin[j]:cObj$end[j])){
					abline(h=i+0.1,col="red")
					abline(h=i-0.1,col="red")
				}
			}
		}
		n <- length(dist[[i]]$Dist)
		points(dist[[i]]$Dist,rep(i,n),col="red")
	}
} 
plot.location  <- function(dist,CytoBand.chr=cytoBand.X, gtObj=gtObj.X, main="Locations"){
#	par(oma=c(1,0,1,0))
	plot(NA, axes = FALSE, xlab = "", ylab = "", xaxs = "i",
      	 main = main, xlim = c(1,5), ylim = c(1,length(dist)))
#  	axis(1)
	for (i in 1:length(dist)){
		pos <- dist[[i]]$Pos
		ind.cyto <- which(CytoBand.chr[,"ePos"] > pos & CytoBand.chr[,"sPos"] <= pos)
#		gene <- get.gene.list2(pos-5,pos+5,DMEL.chr)$gene
		gene <- get.gt.list(pos-5,pos+5,gtObj)$gene
		txt <- paste(dist[[i]]$Pos, " \t(", CytoBand.chr[ind.cyto,"CP"],")\t", gene,sep="")
#		text(x=,y=i, txt,cex=0.5)
		text(x=1,y=i, txt,offset=0,pos=4,cex=0.6)
	}
	box()
}

DrawSequence <- function(sPos,ePos,chr=chr.X, motif="", main="",sub="",nc.row=100){
	num.row <- ceiling((ePos-sPos)/nc.row)
	xlim 	<- c(0,nc.row)
	ylim 	<-  c(0,num.row+1)
	plot(NA, axes = FALSE, xlab = "", ylab = "", xaxs = "i",
	      main = "", xlim = xlim, ylim = ylim)
	box()
	seq <- substr(chr, sPos,ePos)
	seq.letter <- strsplit(seq, split="|")
	seq.col <- rep(1,nchar(seq))
	colors <- c("black","red","orange","green","blue",
			"purple","red4","orange4","green4","blue4","purple4")
	if (length(motif)>0){
		for (i in 1:length(motif)){
			mot	<- motif[i]
			nc1 	<- nchar(mot)-1
			ms <- (matchDNAPattern.good(mot,seq))
			if (length(ms)>0){
				for (j in 1:length(ms)){
					seq.col[(ms[j]-nc1/2):(ms[j]+nc1/2)] <- 2*i
				}
			}
			mot <- reverseComplement(DNAString(mot))
			ms <- (matchDNAPattern.good(mot,seq))
			if (length(ms)>0){
				for (j in 1:length(ms)){
					seq.col[(ms[j]-nc1/2):(ms[j]+nc1/2)] <- 2*i+1
				}
			}

		}
	}

	for (i in 1:nchar(seq)){
		text(x=(i-1)%%nc.row,y=num.row-floor((i-1)/nc.row),seq.letter[[1]][i], col = colors[seq.col[i]],offset=0,pos=4,cex=0.5)	
	}
}

#------------- some function related to IGR ------ 
get.GeneIndices	<- function(gtObj, ngAnnot, FullEndProbes=TRUE)  {
# Some probes may be partially covered by genes
  print("FullEndProbes=TRUE:  Ending probes are fully covered by gene.")
	len	<- length(gtObj$gene)
	sInd	<- eInd	<-	integer(len)
  	res <- list()

  	if (FullEndProbes==TRUE){
	  	for(i in 1:len)  {
			sInd[i]	<- min(which(ngAnnot[,1]>=gtObj$gene.sPos[i]))
			eInd[i]	<- max(which(ngAnnot[,2]<=gtObj$gene.ePos[i]))
  		}
	}else{
	  	for(i in 1:length(gtObj$gene))  {
			sInd[i]	<- min(which(ngAnnot[,2]>=gtObj$gene.sPos[i]))
			eInd[i]	<- max(which(ngAnnot[,1]<=gtObj$gene.ePos[i]))
  		}
	}
	gene.ind	<- cbind(sInd,eInd)
	colnames(gene.ind) <- c("sInd","eInd")
  	rownames(gene.ind) <- gtObj$gene  
return(gene.ind)}

get.IGR	<- function(Ind, GeneIndices, minsize){
#  print("Get IGR")
  s <- integer(length(Ind))
  for (i in 1:nrow(GeneIndices))
	s[GeneIndices[i,1]:GeneIndices[i,2]] <- -1
  cObj <-    get.cutoff.clusters(s, -0.5, minsize)
  return(cObj)}

get.IGRwithBS <- function(cObj.IGR, cObj.BS){
# print("Get IGR with BS isolated from Binding genes")
  k=1
  ind <- list()
  for (i in 1:length(cObj.IGR$size)){
	if (length(intersect(which(cObj.BS$begin >= cObj.IGR$begin[i]), which(cObj.BS$end <= cObj.IGR$end[i])))>0){
	   	ind[[k]]<- i	
		k=k+1
	}
  }
  ind.good <- unlist(ind)
	length(ind.good)
  return(list(size 	= cObj.IGR$size[ind.good],
		  begin 	= cObj.IGR$begin[ind.good],
		  end 	= cObj.IGR$end[ind.good]))	}

get.IGR.BS <- function(cObj.IGR, cObj.BS){
# print("Get IGR BS isolated from Binding Genes")
  k=1
  ind <- list()
  for (i in 1:length(cObj.BS$size)){
	if (length(intersect(which(cObj.IGR$begin <= cObj.BS$begin[i]), which(cObj.IGR$end >= cObj.BS$end[i])))>0){
	   	ind[[k]]<- i	
		k=k+1
	}
  }
  ind.good <- unlist(ind)
	length(ind.good)
  return(list(size 	= cObj.BS$size[ind.good],
		  begin 	= cObj.BS$begin[ind.good],
		  end 	= cObj.BS$end[ind.good]))	}



#=====================================
ToNewIndex	<- function(data){
  if (length(data) == 388536){
	print("Shrink to new index")
	index <- c(index.2L.F,index.4,index.X)	
	data.new	<- data[index]
	return(data.new)
  }else
   {	
	print("No need to change to new index")
   	return(data)
  }
 }

#=======================================
get.cytoBand	<- function(CytoBand.chr,sPos=0,ePos=0){
	ind.cyto <- which(CytoBand.chr[,"ePos"] > sPos & CytoBand.chr[,"sPos"] < ePos)
	return( paste(CytoBand.chr[ind.cyto,"CP"],collapse=","))
}

#--------------------------------------
# need to load some packages
library("Biostrings")
library("matchprobes")

#rc	<- function(p0){return(reverseSeq(complementSeq(p0)))}	# reverse Complement

rc	<- function(p0){
	p1 <- unlist(strsplit(toupper(reverseSeq(p0)),split="|"))
	p2 <- p1
	for (i in 1:length(p1)){
		if (p1[i] %in% c("A","C","G","T")) p2[i] <- complementSeq(p1[i])		
	}
	return(paste(p2,sep="",collapse=""))
}


#---------------------------------
get.iie.counts <- function(cObj, iie=iie.X){
  # get the intergenic,intron, and exon portion of a set of probes
  iie.counts <- list()
  ns	<- length(cObj$size)
  if (ns < 1) {
	print("No cluster")
	return
  }
  for (i in 1:ns){
	iie.counts[[i]] <- list()
 	ind <- cObj$begin[i]:cObj$end[i]
	iie.counts[[i]] <- list("intergenic"	= length(intersect(ind,iie$ind.intergenic)),
					"intron"		= length(intersect(ind,iie$ind.intron)),
					"exon"	 	= length(intersect(ind,iie$ind.exon)))
  }
  return(iie.counts)
 }

get.iie.counts.X <- function(cObj){
  return(get.iie.counts(cObj,iie.X))
}

get.iie.counts.2L <- function(cObj){
  return(get.iie.counts(cObj,iie.2L))
}


#-----------
get.cObj.table0 <- function(data, cutoff=0, minsizes=NULL){
  
  if (length(data) != 388468){
    print("data length is not 388468")
    return()
  }
  
  if (is.null(minsizes))  minsizes <- 5:15
  print(minsizes)
  cObj.table <- matrix(0, nrow=length(minsizes),ncol=3)
  colnames(cObj.table) <-
    c("minsize",  paste("Clusters (chr.", c("X","2L"), ")",sep=""))
  
  minsize0 <- min(minsizes)
  
  cObj <- get.cObj(data,cutoffs=cutoff,minsize0)
  for (kk in 1:length(minsizes)){
    minsize1 <- minsizes[kk]
    cObj.table[kk,] <- c(minsize1,
                         length(which(cObj$c.X$size >= minsize1)),
                         length(which(cObj$c.2L$size >= minsize1)))
  }
  return(cObj.table)
}                       


#--------- get cObj for smoothed data -------
get.cObj <- function(data, cutoffs, minsize,index.list=NULL){
  if (!is.null(index.list)){
#    print("call get.cObj.new")
    return(    get.cObj.new(data, cutoffs, minsize,index.list))
  }
  
  #--- ind.X & ind.2L ----
  if (is.matrix(data)){		
    n <- ncol(data)
    if (n!=length(cutoffs)){
      print("ncol(data) != length(cutoffs)")
      return()
    }
    c.X <- c.2L <- list()
    data.X <- data[ind.X,]
    data.2L <- data[ind.2L,]
    for (i in 1:n){
      c.X[[i]]  <- get.cutoff.clusters(data.X[,i], cutoffs[i],minsize)
      c.2L[[i]] <- get.cutoff.clusters(data.2L[,i],cutoffs[i],minsize)      
    }
  }else{
      data.X <- data[ind.X]
      data.2L <- data[ind.2L]
      c.X  <- get.cutoff.clusters(data.X, cutoffs,minsize)
      c.2L <- get.cutoff.clusters(data.2L,cutoffs,minsize)
  }
  return(list(c.X = c.X, c.2L =c.2L))
}

get.cObj.new <- function(data, cutoffs, minsize,index.list=NULL){
  if (is.null(index.list)){
    ind.list <- list(ind.X, ind.2L)
  }else{
    ind.list <- index.list
  }
  
  if (is.matrix(data)){		
    n <- ncol(data)
    if (n!=length(cutoffs)){
      print("ncol(data) != length(cutoffs)")
      return()
    }
    
    c.n <- c <- list()
    for (i in 1:n){
      for (j in 1:length(ind.list)){
        c[[j]] <- get.cutoff.clusters(data[ind.list[[j]],i], cutoffs[i],minsize)
      }
      c.n[[i]] <- c[[j]]
    }
    return(list(c.n=c.n,index.list=ind.list))
  }else{
    c <- list()
    for (j in 1:length(ind.list)){
      c[[j]] <- get.cutoff.clusters(data[ind.list[[j]]], cutoffs,minsize)
    }
    return(list(c=c,index.list=ind.list))
  }
}


In.cObj.old	<- function(cObj.A, cObj.B){
	n.A	<- length(cObj.A$size)
	n.B	<- length(cObj.B$size)
	
	Ans	<- rep("No",n.B)

	for (i in 1:n.A){
		pos	<- as.integer((cObj.A$begin[i]+cObj.A$end[i])/2)
		for (j in 1:n.B){
			ind <- (cObj.B$begin[j]):(cObj.B$end[j])
			if (pos %in% ind)  Ans[j] <- "Yes"
		}
	}
	return(Ans)
 }

In.cObj.new	<- function(cObj.A, cObj.B,minsize=3){
  n.A	<- length(cObj.A$size)
  n.B	<- length(cObj.B$size)
  
  Ans	<- rep("No",n.B)
  
  for (j in 1:n.B){
    cObj.B.1 <- list(begin=cObj.B$begin[j],end=cObj.B$end[j],size=cObj.B$size[j])
    cObj.1 <- compare.n.clusters(list(cObj.A,cObj.B.1),CL=minsize ,minfreq=2)
    if (length(cObj.1$size)>0) Ans[j] <- "Yes"
#    if (length(cObj.1$size)>1)
#      print(paste("Cluster #",j, " in 2nd set is overlapped with ", length(cObj.1$size), " clusters in 1st set",sep="") )
  }
  return(Ans)
}

#cObj.wt$c.X -> cObj.A
#cObj.ss$c.X -> cObj.B

In.cObj	<- function(cObj.A, cObj.B,minsize=3){
  n.A	<- length(cObj.A$size)
  n.B	<- length(cObj.B$size)
  
  Ans	<- rep("No",n.B)

  if (n.A*n.B>0){ 
    
    for (i in 1:n.A){
#    pos	<- as.integer((cObj.A$begin[i]+cObj.A$end[i])/2)
      ind.A <- (cObj.A$begin[i]):(cObj.A$end[i])
      for (j in 1:n.B){
        ind.B <- (cObj.B$begin[j]):(cObj.B$end[j])
#      if (pos %in% ind)  Ans[j] <- "Yes"
        if (length(intersect(ind.A,ind.B))>=minsize)  Ans[j] <- "Yes"
      }
    }
  }
  return(Ans)
}

In.cObj.AB <- function(cObj.A, cObj.B,minsize=3){
  Ans.B <- In.cObj(cObj.A, cObj.B,minsize)
  Ans.A <- In.cObj(cObj.B, cObj.A,minsize)

  print("1st in 2nd:")
  print(table(Ans.A))
  print("2nd  in 1st :")
  print(table(Ans.B))
  return(list(Ans.1in2 = Ans.A, Ans.2in1 =Ans.B))
}

setdiff.cObj <- function(cObj.A, cObj.B,minsize=3){
  if(is.null(cObj.B)) return(cObj.A)
  junk <- In.cObj.AB(cObj.A, cObj.B,minsize)
  ind <- which(junk$Ans.1in2 =="No")
  if (is.null(ind)) return(NULL)
  return(list(begin=cObj.A$begin[ind],
              end  =cObj.A$end[ind],
              size =cObj.A$size[ind]))
}

intersect.cObj <- function(cObj.A, cObj.B,CL=3){
  cObj <- compare.n.clusters(list(cObj.A,cObj.B),CL=3,minfreq=2)
  return(cObj)
}

get.cObj.table <- function(data, sigma, minsizes, cfactors, index.list=NULL){
  minsize.min <- min(minsizes)
  
  table.1 <- matrix(0, nrow=length(minsizes), ncol = length(cfactors))
  colnames(table.1) <- cfactors
  rownames(table.1) <- minsizes
  table.2  <- table.1
  
  if (is.null(index.list))  index.list <-  list(index.X.F,index.2L.F)
  
  for (i in 1:length(cfactors)){
    cutoff <- cfactors[i]*sigma
    cObj <- get.cObj(data,cutoff, minsize.min, index.list=index.list)
    for (j in 1:length(minsizes)){
      table.1[j,i] <- length( which(cObj$c[[1]]$size>=minsizes[j]))
      table.2[j,i] <- length( which(cObj$c[[2]]$size>=minsizes[j]))      
    }    
  }
  ratio <- round( table.2/table.1, 3)

  table <- matrix(paste(ratio, " (", table.2, " : ", table.1, ")" ,sep=""), nrow=length(minsizes),byrow=FALSE)

  colnames(table) <- cfactors
  rownames(table) <- minsizes
  table
  return(table)
}

#----- get the location of maximum density on histogram --

hist.max <- function(M,delta=0.1){
  breaks <- seq(min(M)-delta,max(M)+delta,delta)
  h <- hist(M,plot=FALSE,breaks)
  M.h.max <- h$mids[which.max(h$density)]
  return(M.h.max)
}

density.max <- function(M){
  d <- density(M)
  M.d.max <- d$x[which.max(d$y)]
  return(M.d.max)
}

draw.hist <- function(data, sigma=NULL, index.list=NULL,breaks=NULL,
                      main=NULL,cex.main=1,legend=NULL,cols=NULL,ylim=NULL,xlim=NULL,
                      plot.sigma=TRUE){
  h <- list()
  if (is.null(index.list)){
    index.list <- list(index.X.F,index.2L.F)
  }
  
  if (is.null(sigma)){
    sigma <- mad.diff_lag(data)
  }
  s.d <- sd(data)
  if (is.null(breaks)){
    breaks <- seq(floor(min(data)),ceiling(max(data)),0.1)
  }
  
  y.max <- 0

  for (i in 1:length(index.list)){
    h[[i]] <- hist(data[index.list[[i]]],plot=FALSE,breaks=breaks)
    y.max <- max(y.max,h[[i]]$counts)
  }

  if (is.null(ylim))
    ylim <- c(0,y.max)

  if (is.null(cols))
    cols <- c("red","blue")

  if (is.null(xlim))  xlim <- range(breaks)
#  range(h[[1]]$mids)  

  i=1; plot(h[[i]]$mids,h[[i]]$counts,type="l", pch=".",
    xlab="M (log ratio)",ylab="counts",col=cols[i],xlim = xlim,ylim=ylim,
    main=main,cex.main=cex.main)

#  l.sigma <- paste(expression(sigma^*),": ", round(sigma,3), sep="")
  
  l.sigma <- paste("sigma*",": ", round(sigma,3), sep="")
  l.sd <- paste("s.d.  : ", round(s.d,3), sep="")

  text.col=c("red", "blue", "green", "black")
  text.col[1:2] <- cols[1:2]
  if(is.null(legend)) {
    legend = c("chr.X", "chr.2L")
    text.col <- cols[1:2]
  }else{
    if (length(legend) >= length(index.list)){
      legend = legend[1:length(index.list)]
    }else{
      print("# legend is not enough")
      return()
    }
    if (length(cols) >= length(index.list)){
      cols = cols[1:length(index.list)]
    }else{
      print("# cols is not enough")
      return()
    }
    text.col=cols
  }

  abline(h=0,col="grey")
  abline(v=0,col=1,lty=2)

  if (plot.sigma == TRUE){
    legend <- c(legend, l.sigma,l.sd)
    text.col <- c(text.col, "green","black")
    for (i in 1:4){
      abline(v=i*sigma,col="green",lty=2)
      abline(v=-i*sigma,col="green",lty=2)
    }
  }

  legend("topright",legend=legend,text.col=text.col)
  
  if (length(index.list)>1){
    for (i in 2:length(index.list)){
#      points(h[[i]]$mids,h[[i]]$counts,type="l",pch=".",col=2*i)
      points(h[[i]]$mids,h[[i]]$counts,type="l",pch=".",col=cols[i])
    }
  }

  return()
}


draw.density <- function(data, sigma=NULL, index.list=NULL,main=NULL){
  d<- list()
  if (is.null(index.list)){
    index.list <- list(index.X.F,index.2L.F)
  }
  if (is.null(sigma)){
    sigma <- mad.diff_lag(data)
  }
  y.max <- 0
  for (i in 1:length(index.list)){
    d[[i]] <- density(data[index.list[[i]]])
    y.max <- max(y.max,d[[i]]$y)
  }
  ylim <- c(0,y.max)  
#  i=1; plot(d[[i]],type="b",xlab="M (log ratio)",ylab="density",col="red")
   i=1; plot(d[[i]]$x,d[[i]]$y,xlab="M (log ratio)",ylab="density",col="red",ylim=ylim,pch=".",main=main)
  if (length(index.list)>1){
    for (i in 2:length(index.list)){
      points(d[[i]]$x,d[[i]]$y,col=2*i,type="l",pch=".")
      #points(h[[i]]$mids,h[[i]]$density,type="b", col=2*i)
    }
  }
  abline(v=0,col=1,lty=2)  

  for (i in 1:4){
    abline(v=i*sigma,col="green",lty=2)
    abline(v=-i*sigma,col="green",lty=2)
  }
  return()
}

get.C.sizes <- function(cObj, sizes=NULL){

  if (is.null(sizes)) sizes <- seq(400,1000,100)   # 40kb to 100kb

  c.sizes <- numeric(length(sizes))
  diff.b <- diff(cObj$begin)  
  for (i in 1:length(sizes)){
    size <- sizes[i]
    c.sizes[i] <- length(which(diff.b>=size))
  }
  table <- cbind(sizes,c.sizes+1)
  colnames(table) <- c("Resolution(in probes)",
                       "# clusters ")
  return(table)
}

merg.cObj <- function(cObj,min.gap=5){
  nc <- length(cObj$size)
  if (nc <2) {
    print("Not even 2 clusters !!")
    return(cObj)
  }
  begin <- sort(cObj$begin)
  end <- sort(cObj$end)
  for (i in nc:2){
    g1   <- begin[i]
    g0   <- end[i-1]
    if ((g1-g0)<=min.gap){
      begin <- begin[-i]
      end <- end[-(i-1)]
    }
  }
  size <- end-begin+1
  return(list(begin=begin,end=end,size=size))  
}

Cutoff.top <- function(data,top0=0.30, iter=20, minsize=8, minfreq=2, CL=3, err=0.10){
  nc <-ng <- top.list <- numeric(iter)
  if (ncol(data) <2) {
    print("not enough data")
    return()
  }
  top <- top0
  i=1
  for (i in 1:iter){
    top.list[i] <- top
    (cutoffs <- Cutoff_quantile(sm(data[index.X.F,]),top))
    cObj <- get.cObj(sm(data),cutoffs,minsize=minsize)
    cObj.X.common <- compare.n.clusters(cObj$c.X,CL=CL,minfreq=minfreq)
    nc[i] <- length(cObj.X.common$size)
    (top <-   (sum(cObj.X.common$size)/length(index.X.F))/(1-err))
    bg <- get.genes.cObj(cObj.X.common)
    ng[i] <- length(bg)
#    if (ng[i] < 5) i=iter+1
  }
  table <- cbind(top.list,nc,ng)
  colnames(table) <- c("Cutoff Quantile(chr.X)", "Clusters (Chr.X)",
                       "Bound genes (Chr.X)")
  return(table)
}


get.PercentOverlap2 <- function(data,minsize=8, tops=seq(0.05,0.20,0.15)){
#  print("only consider the chr.X")
  if (!is.matrix(data)){
    print("data needs to be a matrix")
    return()
  }
  if (ncol(data) !=2){
    print("data needs to be a matrix of 2 colnames")
    return()
  }
  
  ntop <- length(tops)
  cp <- matrix(0,nrow=ntop, ncol=2)
  cp[,1] <- tops
  g <- cp
  for (j in 1:ntop){
    top <- tops[j]
    cutoffs <- Cutoff_quantile(sm(data),top)
    cObj <- get.cObj(data,cutoffs,minsize)
    names(cObj$c.X)
    cObj.1 <- compare.n.clusters(cObj$c.X,CL=3,minfreq=1)
    cObj.2 <- compare.n.clusters(cObj$c.X,CL=3,minfreq=2)
#    genes.1 <- get.genes.cObj(cObj.1)
#    genes.2 <- get.genes.cObj(cObj.2)
    cp[j,2] <-    sum(cObj.2$size)/   sum(cObj.1$size)
#    g[j,2] <- length(genes.2)/length(genes.1)
  }
  return(list(ClusteredProbes=cp,Genes=g ))
}

get.PercentOverlap <- function(data,minsize=8, tops=seq(0.05,0.20,0.15)){
  if (!is.matrix(data)){
    print("data needs to be a matrix")
    return()
  }
  nc <- ncol(data)

  ntop <- length(tops)
  p.a <- matrix(0,nrow=ntop,ncol=nc)
  colnames(p.a)    <- colnames(data)
  colnames(p.a)[1] <- "quantile cutoff"

  p.a[,1] <- tops
  g.a <- p.a
  for (i in 2:nc){
    data0 <- data[,c(1,i)]
    m <- get.PercentOverlap2(data0, minsize,tops)
    p.a[,i] <- m$ClusteredProbes[,2]
    g.a[,i] <- m$Genes[,2]
  }
  return(list(ClusteredProbes=p.a,Genes=g.a))
}

#=========== iie ------------

get.iie.table.0 <- function(cObj,iie=iie.X, cytoBand = cytoBand.X, ngAnnot=ngAnnot.X){
  cObj.iie <- get.iie.counts(cObj,iie)
  nr <- length(cObj$size) # number of row for table
  table <- matrix(0,nrow=nr,ncol=4)
  colnames(table) <- c("Seq#", "cytoband", "probes on gene", "intergenic probes")
  
  library("Biostrings")
  for (i in 1:nr){
    table[i,1] <- i
    table[i,2] <- get.cytoBand(cytoBand, 	sPos= ngAnnot[cObj$begin[i],1],
                                     ePos = ngAnnot[cObj$end[i],2]-1)
    table[i,3] <- cObj.iie[[i]]$intron+cObj.iie[[i]]$exon
    table[i,4] <- cObj.iie[[i]]$intergenic
  }    
  return(table)
}

get.iie.table.X.0 <- function(cObj){
 return(get.iie.table.0(cObj,iie=iie.X, cytoBand = cytoBand.X, ngAnnot=ngAnnot.X))
}

get.iie.table.2L.0 <- function(cObj){
 return(get.iie.table.0(cObj,iie=iie.2L, cytoBand = cytoBand.2L, ngAnnot=ngAnnot.2L))
}

get.iie.table <- function(cObj,iie=iie.X){
  cObj.iie <- get.iie.counts(cObj, iie)
  table <- matrix(unlist(cObj.iie),ncol=3,byrow=TRUE)
  colnames(table) <-  paste("probes @ ", c("IGR", "introns", "exons"),sep="")
  table.sum <- colSums(table)
  percentage <- round(table.sum/sum(table.sum),3)*100
  junk <- paste(table.sum, " (", percentage, "% )",sep="")
  names(junk ) <- names(table.sum)
  return(list(all=table,sum=table.sum,percetage=percentage,summary=junk))
}

get.iie.table.X <- function(cObj){
  return(get.iie.table(cObj,iie=iie.X))
}

get.iie.table.2L <- function(cObj){
  return(get.iie.table(cObj,iie=iie.2L))
}

#----------------
range0 <- function(x){
  return(c(ceiling(min(x)),floor(max(x))))
}

range1 <- function(x){
  return(c(floor(min(x)),ceiling(max(x))))
}

#----------- shift LR based on 2L data, normalize it according 3*sigma^* ===========
norm <- function(LR, fc=3){
  for (i in 1:ncol(LR)) {
    print(density.max(LR[index.2L.F,i]))
    LR[,i] <- LR[,i]-density.max(LR[index.2L.F,i])
  }
  cutoffs <- fc*mad.diff_lag(LR)
  for (i in 1:ncol(LR))  LR[,i] <- LR[,i]/cutoffs[i]
  return(LR)
}


#------------- get Index from Position ---
get.Ind.Pos <- function(Pos, ngAnnot=ngAnnot.X.F){
  return(max(which(ngAnnot[,1]<Pos)))
}
get.Ind.Pos.X <- function(Pos){
  return(get.Ind.Pos(Pos,ngAnnot=ngAnnot.X.F))
}
get.Ind.Pos.2L <- function(Pos){
  return(get.Ind.Pos(Pos,ngAnnot=ngAnnot.2L.F))
}

